
//		HtmlDecode.cpp - reimplements the @HTMLDECODE and @HTMLENCODE functions.


#define UNICODE						1
#define _UNICODE					1

#include <stdio.h>
#include <tchar.h>

#include <windows.h>

#include "PlugIn.h"
#include "TakeCmd.h"


#define VER_MAJOR					1
#define VER_MINOR					0
#define VER_BUILD					2
#define VER_PATCH					1

#define PLUGIN_NAME					L"HtmlDecode"

#ifdef _WIN64
#define PLATFORM					L"x64"
#else
#define PLATFORM					L"x86"
#endif

#define MAX_ARG_LEN					8192
#define SHORT_BUF_LEN				1024

#define ERRHANDLE					GetStdHandle( STD_ERROR_HANDLE )


#define MACRO_EXP( arg )			#arg
#define MACRO_STR( arg )			MACRO_EXP( arg )



const wchar_t PluginFeatures[] = L"@htmldecode,@htmlencode";


const wchar_t F_HtmlDecodeHelpText[]	= L"@HTMLDECODE[string] \n\
\n\
Replaces HTML entities in a string.\n\
\n\
@HTMLDECODE recognizes all HTML 4 character entities,\u00a0\u00a0\n\
including &#nnn; and &#xnnnn; numeric entities. \n\
\n\
Note that the returned string may contain ampersands,\n\
pipes, or redirection operators.  You can deal with this\n\
by quoting the return value, or through liberal use of\n\
SETDOS /X to disable features.\n";


const wchar_t F_HtmlEncodeHelpText[]	= L"@HTMLENCODE[string] \n\
\n\
Replaces characters in a string with HTML entities.\n\
\n\
Note that the returned string is very likely to contain\n\
ampersands.  You can deal with this by quoting the return\u00a0\u00a0\n\
value, or by using SETDOS to change or disable the\n\
command separator character.\n";



#include "HtmlEntities.cpp"



int DisplayErrorHeader()
{
	wchar_t shortbuf[MAX_ARG_LEN] = L"%_batch";

	Qprintf( ERRHANDLE, L"%s plugin: ", PLUGIN_NAME );

	ExpandVariables( shortbuf, 1 );
	if ( shortbuf[0] == '0' )
		return 0;

	wcscpy_s( shortbuf, MAX_ARG_LEN, L"%_batchname [%_batchline]  " );
	ExpandVariables( shortbuf, 1 );
	Qprintf( ERRHANDLE, shortbuf );

	return 0;
}



BOOL APIENTRY DllMain( HANDLE hModule, DWORD  dwReason, LPVOID lpReserved )
{
	switch( dwReason )	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
	}
	return TRUE;
}



DLLExports LPPLUGININFO WINAPI GetPluginInfo( void )
{
	static PLUGININFO piInfo;

	piInfo.pszDll			= (LPTSTR) PLUGIN_NAME;
	piInfo.pszAuthor		= (LPTSTR) L"Charles Dye";
	piInfo.pszEmail			= (LPTSTR) L"cdye@unm.edu";
	piInfo.pszWWW			= (LPTSTR) L"http://charlesdye.net/plugins/htmldecode.html";
	piInfo.pszDescription	= (LPTSTR) L"Reimplements the @HTMLDECODE and @HTMLENCODE functions.";
	piInfo.pszFunctions		= (LPTSTR) PluginFeatures;
	piInfo.nMajor			= VER_MAJOR;
	piInfo.nMinor			= VER_MINOR;
	piInfo.nBuild			= VER_BUILD;

	return &piInfo;
}



DLLExports BOOL WINAPI InitializePlugin( void )
{
	wchar_t shortbuf[SHORT_BUF_LEN] = L"%[_transient]%[_pipe]%[noloadmsg]";
	ExpandVariables( shortbuf, 1 );

	if ( !wcscmp( shortbuf, L"00" ) )
		Printf( L"%s v%u.%u.%u loaded.\r\n", PLUGIN_NAME, VER_MAJOR, VER_MINOR, VER_BUILD );

	return 0;
}



DLLExports BOOL WINAPI ShutdownPlugin( BOOL bEndProcess )
{
	return 0;
}



DLLExports int WINAPI f_htmldecode( LPTSTR String )
{
	if ( !String )
		return 1;


	int rv = Deentitize( String, MAX_ARG_LEN );


	if ( rv == -2 ) {
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"The resulting string is too long.\r\n" );
		return -1;
	}
	else if ( rv == -3 ) {
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"Can\u2019t allocate temp buffer.\r\n" );
		return -1;
	}

	return 0;
}



DLLExports int WINAPI f_htmlencode( LPTSTR String )
{
	if ( !String )
		return 1;


	int rv = Entitize( String, MAX_ARG_LEN );


	if ( rv == -2 ) {
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"The resulting string is too long.\r\n" );
		return -1;
	}
	else if ( rv == -3 ) {
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"Can\u2019t allocate temp buffer.\r\n" );
		return -1;
	}

	return 0;
}



/*
DLLExports int WINAPI entities( LPTSTR String )
{
	if ( !String )
		return 1;


	for ( int i = 0; i < NUM_HTML_ENTITIES; i++ )
		Printf( L"U+%04X   %5u    '%c'    &%s;\r\n", HtmlEntity[i].Char, HtmlEntity[i].Char, HtmlEntity[i].Char, HtmlEntity[i].Name );

	return 0;
}
*/



const int NumHelpTexts							= 2;
const wchar_t HelpFeature[NumHelpTexts][12]		= { L"@htmldecode", L"@htmlencode" };
const wchar_t *HelpText[NumHelpTexts]			= { F_HtmlDecodeHelpText, F_HtmlEncodeHelpText };



//		Simpler MessageBox() help, useful for smaller plugins which don't merit a full HTML help system:

DLLExports int WINAPI Help( LPTSTR inString )
{
	wchar_t WindowTitle[SHORT_BUF_LEN]		= L"";


	wsprintf( WindowTitle, L"%s Plugin Help", PLUGIN_NAME );

	for ( int i = 0; i < NumHelpTexts; i++ ) {
		if ( !_wcsicmp( inString, HelpFeature[i] ) ) {
			MessageBox( GetForegroundWindow(), HelpText[i], WindowTitle, MB_OK );
			return 1;
		}
	}

	return 0;
}
